@@ -72,7 +72,7 @@ class distaz:
     and I vaguely remember a perl port. Long live distaz! 
 
     Mijian Xu, Jan 01, 2016
-    Add np.ndarray to availible input varible.
+    Add np.ndarray to available input.
     """
 
     def __init__(self, lat1, lon1, lat2, lon2):
@@ -153,7 +153,7 @@ class distaz:
 
         dbaz_idx = np.where(dbaz < 0.0)[0]
         if len(dbaz_idx) != 0:
-            if isinstance(dbaz, (int, float)):
+            if isinstance(dbaz, (int, float, np.integer, np.floating)):
                 dbaz += 2 * math.pi
             else:
                 dbaz[dbaz_idx] += 2 * math.pi
