@@ -11,12 +11,21 @@ def _cat2df(cat):
 
 
 class Query():
-    def __init__(self, server='IRIS'):
-        self.client = Client(server)
+    def __init__(self, server='IRIS', **kwargs):
+        self.client = Client(server, **kwargs)
 
     def get_events(self, starttime=None,
                    endtime=UTCDateTime.now(), 
                    **kwargs):
+        """Get events from IRIS
+
+        :param starttime: Start time of events, defaults to None
+        :type starttime: :class:`obspy.UTCDateTime`, optional
+        :param endtime: End time of events, defaults to UTCDateTime.now()
+        :type endtime: :class:`obspy.UTCDateTime`, optional
+        :return: Events
+        :rtype: :class:`obspy.Catalog`
+        """
         if endtime > UTCDateTime.now():
             endtime = UTCDateTime.now()
         events = Catalog()
@@ -27,9 +36,17 @@ class Query():
         else:
             chunk_length = 365 * 86400
             while starttime <= endtime:
-                events += self.client.get_events(starttime=starttime,
-                                                endtime=starttime + chunk_length,
+                if endtime-starttime < chunk_length:
+                    nowtime = endtime
+                else:
+                    nowtime=starttime + chunk_length
+                try:
+                    events += self.client.get_events(starttime=starttime,
+                                                endtime=nowtime,
                                                 orderby='time-asc', **kwargs)
+                except:
+                    starttime += chunk_length
+                    continue
                 if starttime + chunk_length > endtime:
                     chunk = endtime - starttime
                     if chunk <= 1:
