@@ -10,8 +10,9 @@ from seispy.io import Query, _cat2df
 from seispy.para import RFPara
 from seispy import distaz
 from seispy.eq import EQ
-from seispy.setuplog import setuplog
+from seispy.setuplog import SetupLog
 from seispy.catalog import read_catalog_file
+from seispy.utils import scalar_instance
 import glob
 import numpy as np
 from datetime import timedelta
@@ -70,7 +71,8 @@ def datestr2regex(datestr):
 
 
 def read_catalog(logpath:str, b_time, e_time, stla:float, stlo:float,
-                 magmin=5.5, magmax=10., dismin=30., dismax=90.):
+                 magmin=5.5, magmax=10., dismin=30., dismax=90.,
+                 depthmin=0, depthmax=800):
     """Read local catalog with seispy or QUAKEML format
 
     :param logpath: Path to catalogs
@@ -102,6 +104,7 @@ def read_catalog(logpath:str, b_time, e_time, stla:float, stlo:float,
     dis = distaz(stla, stlo, eq_lst['evla'], eq_lst['evlo']).delta
     eq_lst = eq_lst[(eq_lst['date']>=b_time) & (eq_lst['date']<=e_time) & \
                     (eq_lst['mag']>=magmin) & (eq_lst['mag']<=magmax) & \
+                    (eq_lst['evdp']>=depthmin) & (eq_lst['evdp']<=depthmax) & \
                     (dis>=dismin) & (dis<=dismax)]
     return eq_lst
 
@@ -109,9 +112,12 @@ def read_catalog(logpath:str, b_time, e_time, stla:float, stlo:float,
 def fetch_waveform(eq_lst, para, model, logger):
     tb = np.max([2*para.noiselen, 2*para.time_before])
     te = np.max([2*para.noiselen, 2*para.time_after])
-    query = Query(para.data_server)
+    try:
+        query = para.stainfo.query
+    except:
+        logger.RFlog.error('Please load station information and search earthquake before fetch waveform')
     new_col = ['dis', 'bazi', 'data', 'datestr']
-    eq_match = pd.DataFrame(columns=new_col)
+    eqall = []
     for i, row in eq_lst.iterrows():
         datestr = row['date'].strftime('%Y.%j.%H.%M.%S')
         daz = distaz(para.stainfo.stla, para.stainfo.stlo, row['evla'], row['evlo'])
@@ -143,7 +149,11 @@ def fetch_waveform(eq_lst, para, model, logger):
             continue
         this_eq.get_time_offset(row['date'])
         this_df = pd.DataFrame([[daz.delta, daz.baz, this_eq, datestr]], columns=new_col, index=[i])
-        eq_match = pd.concat([eq_match, this_df])
+        eqall.append(this_df)
+    if not eqall:
+        logger.RFlog.error('No waveforms fetched')
+        sys.exit(1)
+    eq_match = pd.concat(eqall)
     ind = eq_match.index.drop_duplicates(keep=False)
     eq_match = eq_match.loc[ind]
     return pd.concat([eq_lst, eq_match], axis=1, join='inner')
@@ -151,6 +161,31 @@ def fetch_waveform(eq_lst, para, model, logger):
 
 def match_eq(eq_lst, pathname, stla, stlo, logger, ref_comp='Z', suffix='SAC', offset=None,
              tolerance=210, dateformat='%Y.%j.%H.%M.%S'):
+    """Match earthquakes with local SAC files
+
+    :param eq_lst: Earthquake list
+    :type eq_lst: pandas.DataFrame
+    :param pathname: Path to SAC files
+    :type pathname: str
+    :param stla: Station latitude
+    :type stla: float
+    :param stlo: Station longitude
+    :type stlo: float
+    :param logger: Logger
+    :type logger: seispy.setuplog.SetupLog
+    :param ref_comp: Reference component, defaults to 'Z'
+    :type ref_comp: str, optional
+    :param suffix: Suffix of SAC files, defaults to 'SAC'
+    :type suffix: str, optional
+    :param offset: Time offset between SAC files and earthquakes, defaults to None
+    :type offset: float, optional
+    :param tolerance: Tolerance of time offset, defaults to 210
+    :type tolerance: int, optional
+    :param dateformat: Date format of SAC files, defaults to '%Y.%j.%H.%M.%S'
+    :type dateformat: str, optional
+    :return: Earthquake list with matched SAC files
+    :rtype: pandas.DataFrame
+    """
     pattern = datestr2regex(dateformat)
     ref_eqs = glob.glob(join(pathname, '*{0}*{1}'.format(ref_comp, suffix)))
     if len(ref_eqs) == 0:
@@ -172,7 +207,7 @@ def match_eq(eq_lst, pathname, stla, stlo, logger, ref_comp='Z', suffix='SAC', o
         else:
             raise TypeError('offset should be int or float type')
     new_col = ['dis', 'bazi', 'data', 'datestr']
-    eq_match = pd.DataFrame(columns=new_col)
+    eq_match_lst = []
     for datestr, b_time, offs in sac_files:
         date_range_begin = b_time + timedelta(seconds=offs - tolerance)
         date_range_end = b_time + timedelta(seconds=offs + tolerance)
@@ -187,7 +222,11 @@ def match_eq(eq_lst, pathname, stla, stlo, logger, ref_comp='Z', suffix='SAC', o
         this_eq.get_time_offset(results.iloc[0]['date'])
         daz = distaz(stla, stlo, results.iloc[0]['evla'], results.iloc[0]['evlo'])
         this_df = pd.DataFrame([[daz.delta, daz.baz, this_eq, datestr]], columns=new_col, index=results.index.values)
-        eq_match = pd.concat([eq_match, this_df])
+        eq_match_lst.append(this_df)
+    if not eq_match_lst:
+        logger.RFlog.error('No earthquakes matched')
+        sys.exit(1)
+    eq_match = pd.concat(eq_match_lst)
     ind = eq_match.index.drop_duplicates(keep=False)
     eq_match = eq_match.loc[ind]
     return pd.concat([eq_lst, eq_match], axis=1, join='inner')
@@ -206,7 +245,7 @@ def CfgModify(cfg_file, session, key, value):
 class RF(object):
     def __init__(self, cfg_file=None, log=None):
         if log is None:
-            self.logger = setuplog()
+            self.logger = SetupLog()
         else:
             self.logger = log
         if cfg_file is None:
@@ -242,12 +281,23 @@ class RF(object):
     def date_end(self, value):
         self.para.date_end = value
 
-    def load_stainfo(self):
+    def load_stainfo(self, use_date_range=True):
         try:
             if self.para.use_remote_data:
                 self.logger.RFlog.info('Load station info of {}.{} from {} web-service'.format(
                     self.para.stainfo.network, self.para.stainfo.station, self.para.data_server))
-                self.para.stainfo.get_station_from_ws(self.para.data_server)
+                self.para.stainfo.link_server(
+                    self.para.data_server,
+                    self.para.data_server_user,
+                    self.para.data_server_password
+                )
+                if use_date_range:
+                    self.para.stainfo.get_station_from_ws(
+                        starttime=self.para.date_begin,
+                        endtime=self.para.date_end
+                    )
+                else:
+                    self.para.stainfo.get_station_from_ws()
                 try:
                     self.para._check_date_range()
                 except Exception as e:
@@ -256,7 +306,7 @@ class RF(object):
             else:
                 self.logger.RFlog.info('Load station info from {0}'.format(self.para.datapath))
                 self.para.stainfo.load_stainfo(self.para.datapath, self.para.ref_comp, self.para.suffix)
-            self.logger.RFlog.info('{}/{}, latitude: {:.3f}, longiture: {:.3f}'.format(
+            self.logger.RFlog.info('{}.{}, latitude: {:.3f}, longitude: {:.3f}'.format(
                 self.para.stainfo.network, self.para.stainfo.station, self.stainfo.stla, self.stainfo.stlo))
         except Exception as e:
             self.logger.RFlog.error('Error in loading station info: {0}'.format(e))
@@ -272,7 +322,9 @@ class RF(object):
                 query.get_events(starttime=self.para.date_begin, endtime=self.para.date_end,
                                  latitude=self.para.stainfo.stla, longitude=self.para.stainfo.stlo,
                                  minmagnitude=self.para.magmin, maxmagnitude=self.para.magmax,
-                                 minradius=self.para.dismin, maxradius=self.para.dismax, catalog=catalog)
+                                 minradius=self.para.dismin, maxradius=self.para.dismax, 
+                                 mindepth=self.para.depthmin, maxdepth=self.para.depthmax,
+                                 catalog=catalog)
                 self.eq_lst = query.events
             except Exception as e:
                 raise ConnectionError(e)
@@ -284,13 +336,16 @@ class RF(object):
                 self.eq_lst = read_catalog(self.para.catalogpath, self.para.date_begin, self.para.date_end,
                                            self.para.stainfo.stla, self.para.stainfo.stlo,
                                            magmin=self.para.magmin, magmax=self.para.magmax,
-                                           dismin=self.para.dismin, dismax=self.para.dismax)
+                                           dismin=self.para.dismin, dismax=self.para.dismax,
+                                           depthmin=self.para.depthmin, depthmax=self.para.depthmax)
             except Exception as e:
                 self.logger.RFlog.error('{0}'.format(e))
                 raise e
         self.logger.RFlog.info('{} earthquakes are found'.format(self.eq_lst.shape[0]))
 
     def match_eq(self):
+        """Assosiate earthquakes with local data file or online data.
+        """
         try:
             if self.para.use_remote_data:
                 self.logger.RFlog.info('Fetch seismic data from {}'.format(self.para.data_server))
@@ -422,14 +477,17 @@ class RF(object):
                                        row['data'].datestr, row['data'].inc_correction))
         self.eqs.drop(drop_idx, inplace=True)
 
-    def drop_eq_snr(self, length=None):
+    def drop_eq_snr(self, length=None, z_only=False):
         if length is None:
             length = self.para.noiselen
         self.logger.RFlog.info('Reject data record with SNR less than {0}'.format(self.para.noisegate))
         drop_lst = []
         for i, row in self.eqs.iterrows():
             snr_E, snr_N, snr_Z = row['data'].snr(length=length)
-            mean_snr = np.mean([snr_E, snr_N, snr_Z])
+            if z_only:
+                mean_snr = snr_Z
+            else:
+                mean_snr = np.mean([snr_E, snr_N, snr_Z])
             if mean_snr < self.para.noisegate:
                 drop_lst.append(i)
         self.eqs.drop(drop_lst, inplace=True)
@@ -444,11 +502,11 @@ class RF(object):
     def pick(self, prepick=True, stl=5, ltl=10):
         if prepick:
             self.logger.RFlog.info('Pre-pick {} arrival using STA/LTA method'.format(self.para.phase))
-            for _, row in self.eqs.iterrows():
-                row['data'].phase_trigger(self.para.time_before, self.para.time_after,
-                                          stl=stl, ltl=ltl)
-        self.logger.RFlog.info('{0} events left after virtual checking'.format(self.eqs.shape[0]))
+        for _, row in self.eqs.iterrows():
+            row['data'].phase_trigger(self.para.time_before, self.para.time_after,
+                                      prepick=prepick, stl=stl, ltl=ltl)
         pickphase(self.eqs, self.para, self.logger)
+        self.logger.RFlog.info('{0} events left after visual checking'.format(self.eqs.shape[0]))
 
     def deconv(self):
         shift = self.para.time_before
@@ -483,7 +541,7 @@ class RF(object):
         else:
             pass
         good_lst = []
-        if isinstance(self.para.gauss, (int, float)):
+        if scalar_instance(self.para.gauss):
             gauss = self.para.gauss
         elif gauss is None:
             gauss = self.para.gauss[0]
